import { Inject, Injectable, InjectionToken, Injector, NgModule } from '@angular/core';

const EAGER_PROVIDER = new InjectionToken('EAGER_PROVIDER');

function eagerProviderRegistration(token) {
    return {
        provide: EAGER_PROVIDER,
        useValue: token,
        multi: true
    };
}
function eagerProvider(provider) {
    const token = provider.provide || provider;
    return [provider, eagerProviderRegistration(token)];
}
function eagerProviders(...providers) {
    return providers.map(eagerProvider).reduce((a, b) => a.concat(b), []);
}

class EagerProviderLoaderService {
    constructor() {
        this.loadedProviders = new Set();
    }
    loadProviders(injector) {
        // Obtain the set of tokens for the providers which need to be loaded eagerly.
        const tokens = injector.get(EAGER_PROVIDER, []);
        // Filter out all tokens for the providers which have already been loaded.
        // Afterwards use the injector to get each of the remaining provider tokens.
        // This forces the corresponding providers to be instantiated.
        tokens
            .filter((token) => !this.loadedProviders.has(token))
            .forEach((token) => {
            injector.get(token);
            this.loadedProviders.add(token);
        });
    }
}
EagerProviderLoaderService.decorators = [
    { type: Injectable },
];
/** @nocollapse */
EagerProviderLoaderService.ctorParameters = () => [];

function getEagerProviderLoaderService(injector) {
    // Construct the injector stack from the current injector to the root injector.
    const injectorStack = [injector];
    while (injectorStack[injectorStack.length - 1].parent) {
        injectorStack.push(injectorStack[injectorStack.length - 1].parent);
    }
    // Starting at the root injector, descent in the injector tree to find the first injector that provides the EagerProviderLoaderService.
    let loaderService = null;
    while (!loaderService && injectorStack.length) {
        loaderService = injectorStack.pop().get(EagerProviderLoaderService, null);
    }
    return loaderService;
}
class EagerProviderLoaderModule {
    constructor(injector, eagerProviderTokens) {
        if (!eagerProviderTokens) {
            eagerProviderTokens = [];
        }
        // Retrieve the eager provider loader service that is closest to the root injector.
        const eagerProviderLoaderService = getEagerProviderLoaderService(injector);
        // Load the providers marked for eager loading for the current injector.
        eagerProviderLoaderService.loadProviders(injector);
    }
}
EagerProviderLoaderModule.decorators = [
    { type: NgModule, args: [{
                providers: [EagerProviderLoaderService]
            },] },
];
/** @nocollapse */
EagerProviderLoaderModule.ctorParameters = () => [
    { type: Injector, },
    { type: Array, decorators: [{ type: Inject, args: [EAGER_PROVIDER,] },] },
];

export { EAGER_PROVIDER, eagerProviderRegistration, eagerProvider, eagerProviders, EagerProviderLoaderService, EagerProviderLoaderModule };
